"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _exportNames = {
  setDebugMode: true,
  eventSubscription: true,
  multiply: true,
  cleanCacheDir: true,
  mergeVideos: true
};
exports.cleanCacheDir = cleanCacheDir;
exports.eventSubscription = void 0;
exports.mergeVideos = mergeVideos;
exports.multiply = multiply;
exports.setDebugMode = setDebugMode;

var _ = _interopRequireWildcard(require("lodash"));

var _nonSecure = require("nanoid/non-secure");

var _reactNative = require("react-native");

var _rxjs = require("rxjs");

var _type = require("./type");

Object.keys(_type).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _type[key]) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function () {
      return _type[key];
    }
  });
});

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

let debugMode = false;

function setDebugMode(debug) {
  debugMode = debug;
}

const LINKING_ERROR = `The package 'react-native-video-merger' doesn't seem to be linked. Make sure: \n\n` + _reactNative.Platform.select({
  ios: "- You have run 'pod install'\n",
  default: ''
}) + '- You rebuilt the app after installing the package\n' + '- You are not using Expo managed workflow\n';
const VideoMerger = _reactNative.NativeModules.VideoMerger ? _reactNative.NativeModules.VideoMerger : new Proxy({}, {
  get() {
    throw new Error(LINKING_ERROR);
  }

});
const eventEmitter = new _reactNative.NativeEventEmitter(VideoMerger);
const eventSubscription = new _rxjs.BehaviorSubject(undefined);
exports.eventSubscription = eventSubscription;
eventEmitter.addListener('RNVideoSDK:dataFromNative', eventData => {
  if (debugMode) {
    console.log('#[SDK] RNVideoSDK:dataFromNative', eventData);
  }

  eventSubscription.next(eventData);
});
const __cache__ = {
  subscription: {}
};

function multiply(a, b) {
  return VideoMerger.multiply(a, b);
}
/**
 * 清理缓存目录
 * @returns undefined
 */


async function cleanCacheDir() {
  return await VideoMerger.cleanCacheDir({});
}
/**
 * 开始合并视频
 * @param params MergeVideoParams
 * @returns
 */


async function mergeVideos(_params, onProgress) {
  const params = _.cloneDeep(_params);

  if (params.backgroundColor) {
    params.backgroundColor = (0, _reactNative.processColor)(params.backgroundColor);
  }

  let eventDataCache;
  return new Promise(async (resolve, reject) => {
    const subId = (0, _nonSecure.nanoid)();

    const removeSubscription = () => {
      var _cache__$subscriptio;

      (_cache__$subscriptio = __cache__.subscription[subId]) === null || _cache__$subscriptio === void 0 ? void 0 : _cache__$subscriptio.unsubscribe();
      delete __cache__.subscription[subId];
    };

    const sub = eventSubscription.subscribe(eventData => {
      if (!eventData) {
        return;
      }

      const {
        errorCode,
        progress
      } = eventData; // check error

      const hasError = errorCode !== 0;

      if (hasError) {
        // removeSubscription();
        onProgress && onProgress(1); // reject(eventData);

        return;
      } // update progress


      onProgress && onProgress(progress); // check finished

      const isFinished = progress === 1;

      if (isFinished) {// removeSubscription();
        // resolve(eventData);
      }
    });
    __cache__.subscription[subId] = sub;

    try {
      const res = await VideoMerger.mergeVideos(params);

      if (debugMode) {
        console.log('#[SDK] mergeVideos res', res);
      }

      if (!res.outputFilePath && eventDataCache !== null && eventDataCache !== void 0 && eventDataCache.outputFilePath) {
        res.outputFilePath = eventDataCache === null || eventDataCache === void 0 ? void 0 : eventDataCache.outputFilePath;
      }

      resolve(res);
    } catch (_error) {
      const error = _error;
      console.log('###error', error.userInfo);
      reject(error.userInfo);
    }

    removeSubscription();
  });
}
//# sourceMappingURL=index.js.map