import * as _ from 'lodash';
import { nanoid } from 'nanoid/non-secure';
import { NativeEventEmitter, NativeModules, Platform, processColor } from 'react-native';
import { BehaviorSubject } from 'rxjs';
export * from './type';
let debugMode = false;
export function setDebugMode(debug) {
  debugMode = debug;
}
const LINKING_ERROR = `The package 'react-native-video-merger' doesn't seem to be linked. Make sure: \n\n` + Platform.select({
  ios: "- You have run 'pod install'\n",
  default: ''
}) + '- You rebuilt the app after installing the package\n' + '- You are not using Expo managed workflow\n';
const VideoMerger = NativeModules.VideoMerger ? NativeModules.VideoMerger : new Proxy({}, {
  get() {
    throw new Error(LINKING_ERROR);
  }

});
const eventEmitter = new NativeEventEmitter(VideoMerger);
export const eventSubscription = new BehaviorSubject(undefined);
eventEmitter.addListener('RNVideoSDK:dataFromNative', eventData => {
  if (debugMode) {
    console.log('#[SDK] RNVideoSDK:dataFromNative', eventData);
  }

  eventSubscription.next(eventData);
});
const __cache__ = {
  subscription: {}
};
export function multiply(a, b) {
  return VideoMerger.multiply(a, b);
}
/**
 * 清理缓存目录
 * @returns undefined
 */

export async function cleanCacheDir() {
  return await VideoMerger.cleanCacheDir({});
}
/**
 * 开始合并视频
 * @param params MergeVideoParams
 * @returns
 */

export async function mergeVideos(_params, onProgress) {
  const params = _.cloneDeep(_params);

  if (params.backgroundColor) {
    params.backgroundColor = processColor(params.backgroundColor);
  }

  let eventDataCache;
  return new Promise(async (resolve, reject) => {
    const subId = nanoid();

    const removeSubscription = () => {
      var _cache__$subscriptio;

      (_cache__$subscriptio = __cache__.subscription[subId]) === null || _cache__$subscriptio === void 0 ? void 0 : _cache__$subscriptio.unsubscribe();
      delete __cache__.subscription[subId];
    };

    const sub = eventSubscription.subscribe(eventData => {
      if (!eventData) {
        return;
      }

      const {
        errorCode,
        progress
      } = eventData; // check error

      const hasError = errorCode !== 0;

      if (hasError) {
        // removeSubscription();
        onProgress && onProgress(1); // reject(eventData);

        return;
      } // update progress


      onProgress && onProgress(progress); // check finished

      const isFinished = progress === 1;

      if (isFinished) {// removeSubscription();
        // resolve(eventData);
      }
    });
    __cache__.subscription[subId] = sub;

    try {
      const res = await VideoMerger.mergeVideos(params);

      if (debugMode) {
        console.log('#[SDK] mergeVideos res', res);
      }

      if (!res.outputFilePath && eventDataCache !== null && eventDataCache !== void 0 && eventDataCache.outputFilePath) {
        res.outputFilePath = eventDataCache === null || eventDataCache === void 0 ? void 0 : eventDataCache.outputFilePath;
      }

      resolve(res);
    } catch (_error) {
      const error = _error;
      console.log('###error', error.userInfo);
      reject(error.userInfo);
    }

    removeSubscription();
  });
}
//# sourceMappingURL=index.js.map